#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2015 University of Dundee & Open Microscopy Environment.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Conversion utilities for changing between units.
"""
from __future__ import division


from builtins import str
from past.utils import old_div
from builtins import object
class Conversion(object):
    """
    Base-functor like object which can be used for preparing complex
    equations for converting from one unit to another. Primarily these
    classes and static methods are used via code-generation. Sympy-generated
    strings are placed directly into code. If the proper imports are in place,
    then a top-level Conversion (usually of type Add or
    Mul is returned from the evaluation).
    """

    def __init__(self, *conversions):
        self.conversions = conversions

    def __call__(self, original):
        raise NotImplemented()

    def join(self, sym):
        sb = sym.join([str(x) for x in self.conversions])
        return "(%s)" % sb


class Add(Conversion):
    """
    Sums all the Conversion instances which
    are passed in to the constructor.
    """

    def __call__(self, original):
        rv = 0.0
        for c in self.conversions:
            rv += c(original)
        return rv

    def __str__(self):
        return self.join(" + ")


class Int(Conversion):
    """
    Simple representation of a possibly
    very large integer.
    """

    def __init__(self, i):
        if isinstance(i, int):
            self.i = i
        else:
            self.i = float(i)  # Handles big strings

    def __call__(self, original):
        return self.i

    def __str__(self):
        return str(self.i)


class Mul(Conversion):
    """
    Multiplies all the Conversion instances which
    are passed in to the constructor.
    """

    def __call__(self, original):
        rv = 1.0
        for c in self.conversions:
            rv *= c(original)
        return rv

    def __str__(self):
        return self.join(" * ")


class Pow(Conversion):
    """
    Raises the first argument (base) to
    the power of the second (exponent).
    """

    def __init__(self, base, exp):
        self.base = base
        self.exp = exp

    def __call__(self, original):
        return self.base ** self.exp

    def __str__(self):
        return "(%s ** %s)" % (self.base, self.exp)


class Rat(Conversion):
    """
    Divides the first argument (numerator)
    by the second (denominator).
    """

    def __init__(self, n, d):
        self.n = n
        self.d = d

    def unwrap(self, x, original):
        if isinstance(x, (int, float, str)):
            return float(x)
        else:
            return x(original)

    def __call__(self, original):
        n = self.unwrap(self.n, original)
        d = self.unwrap(self.d, original)
        return old_div(float(n), d)

    def __str__(self):
        return "(%s / %s)" % (self.n, self.d)


class Sym(Conversion):
    """
    Represents the variable of the source
    unit and simply returns the original
    value passed to it.
    """

    def __init__(self, s):
        self.s = s

    def __call__(self, original):
        return float(original)

    def __str__(self):
        return "x"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2011-2015 Glencoe Software, Inc. All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#

"""
   chgrp plugin

   Plugin read by omero.cli.Cli during initialization. The method(s)
   defined here will be added to the Cli class for later use.
"""

from omero.cli import CLI, GraphControl, ExperimenterGroupArg
import sys

HELP = """Move data between groups

Move entire graphs of data based on the ID of the top-node.

Examples:

    # In each case move an image to group 101
    omero chgrp 101 Image:1
    omero chgrp Group:101 Image:2
    omero chgrp ExperimenterGroup:101 Image:3
    # In both cases move five images to the group named "My Lab"
    omero chgrp "My Lab" Image:51,52,53,54,56
    omero chgrp "My Lab" Image:51-54,56

    # Move a plate but leave all images in the original group
    omero chgrp 201 Plate:1 --exclude Image

    # Move all images contained under a project
    omero chgrp 101 Project/Dataset/Image:53
    # Move all images contained under two projects
    omero chgrp 101 Project/Image:201,202

    # Do a dry run of a move reporting the outcome if the move had been run
    omero chgrp 101 Dataset:53 --dry-run
    # Do a dry run of a move, reporting all the objects
    # that would have been moved
    omero chgrp 101 Dataset:53 --dry-run --report

"""


class ChgrpControl(GraphControl):

    def cmd_type(self):
        import omero
        import omero.all
        return omero.cmd.Chgrp2

    def _pre_objects(self, parser):
        parser.add_argument(
            "grp", nargs="?", type=ExperimenterGroupArg,
            help="""Group to move objects to""")

    def is_admin(self, client):
        # check if the user currently logged is an admin
        from omero.model.enums import AdminPrivilegeChgrp
        ec = self.ctx.get_event_context()
        return AdminPrivilegeChgrp in ec.adminPrivileges

    def _process_request(self, req, args, client):
        # Retrieve group id
        gid = args.grp.lookup(client)
        if gid is None:
            self.ctx.die(196, "Failed to find group: %s" % args.grp.orig)

        # Retrieve group
        import omero
        try:
            group = client.sf.getAdminService().getGroup(gid)
        except omero.ApiUsageException:
            self.ctx.die(196, "Failed to find group: %s" % args.grp.orig)

        # Check session owner is member of the target group
        uid = self.ctx.get_event_context().userId
        admin = self.is_admin(client)
        ids = [x.child.id.val for x in group.copyGroupExperimenterMap()]
        # check if the user is an admin
        if uid not in ids and not admin:
            self.ctx.die(197, "Current user is not member of group: %s" %
                         group.id.val)

        # Set requests group
        if isinstance(req, omero.cmd.DoAll):
            for request in req.requests:
                if isinstance(request, omero.cmd.SkipHead):
                    request.request.groupId = gid
                else:
                    request.groupId = gid
        else:
            if isinstance(req, omero.cmd.SkipHead):
                req.request.groupId = gid
            else:
                req.groupId = gid

        super(ChgrpControl, self)._process_request(req, args, client)

    def print_detailed_report(self, req, rsp, status):
        import omero
        if isinstance(rsp, omero.cmd.DoAllRsp):
            for response in rsp.responses:
                if isinstance(response, omero.cmd.Chgrp2Response):
                    self.print_chgrp_response(response)
        elif isinstance(rsp, omero.cmd.Chgrp2Response):
            self.print_chgrp_response(rsp)

    def print_chgrp_response(self, rsp):
        if rsp.includedObjects:
            self.ctx.out("Included objects")
            obj_ids = self._get_object_ids(rsp.includedObjects)
            for k in obj_ids:
                self.ctx.out("  %s:%s" % (k, obj_ids[k]))
        if rsp.deletedObjects:
            self.ctx.out("Deleted objects")
            obj_ids = self._get_object_ids(rsp.deletedObjects)
            for k in obj_ids:
                self.ctx.out("  %s:%s" % (k, obj_ids[k]))

try:
    register("chgrp", ChgrpControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("chgrp", ChgrpControl, HELP)
        cli.invoke(sys.argv[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2014 Glencoe Software, Inc. All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Code-generated omero.model.Power implementation,
based on omero.model.PermissionsI
"""


from builtins import str
from past.builtins import basestring
import Ice
import IceImport
IceImport.load("omero_model_Power_ice")
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"

from omero_model_UnitBase import UnitBase
from omero.model.enums import UnitsPower

from omero.conversions import Add  # nopep8
from omero.conversions import Int  # nopep8
from omero.conversions import Mul  # nopep8
from omero.conversions import Pow  # nopep8
from omero.conversions import Rat  # nopep8
from omero.conversions import Sym  # nopep8


class PowerI(_omero_model.Power, UnitBase):

    UNIT_VALUES = sorted(UnitsPower._enumerators.values())
    CONVERSIONS = dict()
    for val in UNIT_VALUES:
        CONVERSIONS[val] = dict()
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 16)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 19)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 17)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 36)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 20)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.MICROWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.NANOWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 33)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.PICOWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 6), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 42)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Int(1000), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.ATTOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 39)), Sym("attow"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 16), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Int(10)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 20)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 13), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 11)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 4)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 5)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 8)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 4), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.MILLIWATT] = \
        Mul(Int(10), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 7), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 17)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 10), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 14)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Int(100)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 22), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 26)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 19), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.CENTIWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 23)), Sym("centiw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 19), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.CENTIWATT] = \
        Mul(Int(1000), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.DECIWATT] = \
        Mul(Int(100), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 17)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 16), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 8)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Int(10)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Int(100)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 5)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 7), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 4), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 10), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 14)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 13), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 11)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.WATT] = \
        Mul(Int(10), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 25), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 23)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 22), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 20)), Sym("decaw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 17), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.CENTIWATT] = \
        Mul(Int(10), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Int(100)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 19)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 14), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 10)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 4)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 7)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 5), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.MILLIWATT] = \
        Mul(Int(100), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 8), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 16)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 11), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 13)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Int(10)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 23), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 25)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 20), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.DECIWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 22)), Sym("deciw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 36), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 20), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 17), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 19), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 33), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.GIGAWATT] = \
        Mul(Pow(10, 9), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 16), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.KILOWATT] = \
        Mul(Pow(10, 15), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.MEGAWATT] = \
        Mul(Pow(10, 12), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 24), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 21), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 27), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.PETAWATT] = \
        Mul(Int(1000), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 30), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.TERAWATT] = \
        Mul(Pow(10, 6), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 18), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 42), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 39), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.EXAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("exaw"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.ATTOWATT] = \
        Mul(Int(1000), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 13)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 16)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 14)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 33)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 17)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.MICROWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.NANOWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.PICOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 9), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 39)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 6), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.FEMTOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 36)), Sym("femtow"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 27), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 11), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 8), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 10), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 24), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 7), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.KILOWATT] = \
        Mul(Pow(10, 6), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.MEGAWATT] = \
        Mul(Int(1000), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 15), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 12), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 18), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 21), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 9), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 33), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 30), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.GIGAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("gigaw"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 20), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 4), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.DECAWATT] = \
        Mul(Int(10), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.DECIWATT] = \
        Mul(Int(1000), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 16)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 17), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 7)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Int(10)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 4)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 8), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 5), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 11), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 13)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 14), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 10)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.WATT] = \
        Mul(Int(100), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 26), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 22)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 23), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.HECTOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 19)), Sym("hectow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 21), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 5), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.DECAWATT] = \
        Mul(Int(100), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 4), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 18), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.HECTOWATT] = \
        Mul(Int(10), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 9), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 6), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 12), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 15), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.WATT] = \
        Mul(Int(1000), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 27), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 24), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.KILOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("kilow"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 24), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 8), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 5), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 7), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 21), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 4), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.KILOWATT] = \
        Mul(Int(1000), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 12), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 9), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 15), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 18), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 6), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 30), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 27), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MEGAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("megaw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 12), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 4)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 7)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 5)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 9), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 8)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.NANOWATT] = \
        Mul(Int(1000), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 6), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 18), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 15), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MICROWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("microw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 15), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Int(10)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 4)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Int(100)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 12), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 5)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.MICROWATT] = \
        Mul(Int(1000), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 6), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 9), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 21), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 18), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.MILLIWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("milliw"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 9), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 7)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 10)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 8)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 6), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 11)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.MICROWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.PICOWATT] = \
        Mul(Int(1000), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 15), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 33)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 12), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.NANOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("nanow"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 33), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 17), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 14), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 16), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 30), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.GIGAWATT] = \
        Mul(Pow(10, 6), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 13), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.KILOWATT] = \
        Mul(Pow(10, 12), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.MEGAWATT] = \
        Mul(Pow(10, 9), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 21), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 18), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 24), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 27), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.TERAWATT] = \
        Mul(Int(1000), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 15), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 39), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 36), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PETAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("petaw"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 6), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 10)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 13)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 11)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Int(1000), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 14)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.MICROWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.NANOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 12), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 36)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 9), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.PICOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 33)), Sym("picow"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 30), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 14), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 11), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 13), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 27), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.GIGAWATT] = \
        Mul(Int(1000), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 10), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.KILOWATT] = \
        Mul(Pow(10, 9), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.MEGAWATT] = \
        Mul(Pow(10, 6), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 18), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 15), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 21), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 24), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 12), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 36), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 33), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.TERAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("teraw"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 18), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.CENTIWATT] = \
        Mul(Int(100), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Int(10)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.DECIWATT] = \
        Mul(Int(10), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 15), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Int(100)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 6), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.MILLIWATT] = \
        Mul(Int(1000), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 9), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 12), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 24), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 21), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.WATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("w"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.ATTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 22)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 25)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 23)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 42)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 33)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 26)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.MICROWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.NANOWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 39)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.PICOWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 36)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 48)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOCTOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 45)), Sym("yoctow"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 42), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 26), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 23), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 25), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.EXAWATT] = \
        Mul(Pow(10, 6), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 39), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.GIGAWATT] = \
        Mul(Pow(10, 15), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 22), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.KILOWATT] = \
        Mul(Pow(10, 21), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.MEGAWATT] = \
        Mul(Pow(10, 18), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 30), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 27), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 33), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.PETAWATT] = \
        Mul(Pow(10, 9), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 36), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.TERAWATT] = \
        Mul(Pow(10, 12), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 24), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 48), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 45), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.YOTTAWATT][UnitsPower.ZETTAWATT] = \
        Mul(Int(1000), Sym("yottaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.ATTOWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.CENTIWATT] = \
        Mul(Rat(Int(1), Pow(10, 19)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.DECAWATT] = \
        Mul(Rat(Int(1), Pow(10, 22)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.DECIWATT] = \
        Mul(Rat(Int(1), Pow(10, 20)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.EXAWATT] = \
        Mul(Rat(Int(1), Pow(10, 39)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.FEMTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 6)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.GIGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 30)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.HECTOWATT] = \
        Mul(Rat(Int(1), Pow(10, 23)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.KILOWATT] = \
        Mul(Rat(Int(1), Pow(10, 24)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.MEGAWATT] = \
        Mul(Rat(Int(1), Pow(10, 27)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.MICROWATT] = \
        Mul(Rat(Int(1), Pow(10, 15)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.MILLIWATT] = \
        Mul(Rat(Int(1), Pow(10, 18)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.NANOWATT] = \
        Mul(Rat(Int(1), Pow(10, 12)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.PETAWATT] = \
        Mul(Rat(Int(1), Pow(10, 36)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.PICOWATT] = \
        Mul(Rat(Int(1), Pow(10, 9)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.TERAWATT] = \
        Mul(Rat(Int(1), Pow(10, 33)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.WATT] = \
        Mul(Rat(Int(1), Pow(10, 21)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.YOCTOWATT] = \
        Mul(Int(1000), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 45)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZEPTOWATT][UnitsPower.ZETTAWATT] = \
        Mul(Rat(Int(1), Pow(10, 42)), Sym("zeptow"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.ATTOWATT] = \
        Mul(Pow(10, 39), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.CENTIWATT] = \
        Mul(Pow(10, 23), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.DECAWATT] = \
        Mul(Pow(10, 20), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.DECIWATT] = \
        Mul(Pow(10, 22), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.EXAWATT] = \
        Mul(Int(1000), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.FEMTOWATT] = \
        Mul(Pow(10, 36), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.GIGAWATT] = \
        Mul(Pow(10, 12), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.HECTOWATT] = \
        Mul(Pow(10, 19), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.KILOWATT] = \
        Mul(Pow(10, 18), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.MEGAWATT] = \
        Mul(Pow(10, 15), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.MICROWATT] = \
        Mul(Pow(10, 27), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.MILLIWATT] = \
        Mul(Pow(10, 24), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.NANOWATT] = \
        Mul(Pow(10, 30), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.PETAWATT] = \
        Mul(Pow(10, 6), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.PICOWATT] = \
        Mul(Pow(10, 33), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.TERAWATT] = \
        Mul(Pow(10, 9), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.WATT] = \
        Mul(Pow(10, 21), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.YOCTOWATT] = \
        Mul(Pow(10, 45), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.YOTTAWATT] = \
        Mul(Rat(Int(1), Int(1000)), Sym("zettaw"))  # nopep8
    CONVERSIONS[UnitsPower.ZETTAWATT][UnitsPower.ZEPTOWATT] = \
        Mul(Pow(10, 42), Sym("zettaw"))  # nopep8
    del val

    SYMBOLS = dict()
    SYMBOLS["ATTOWATT"] = "aW"
    SYMBOLS["CENTIWATT"] = "cW"
    SYMBOLS["DECAWATT"] = "daW"
    SYMBOLS["DECIWATT"] = "dW"
    SYMBOLS["EXAWATT"] = "EW"
    SYMBOLS["FEMTOWATT"] = "fW"
    SYMBOLS["GIGAWATT"] = "GW"
    SYMBOLS["HECTOWATT"] = "hW"
    SYMBOLS["KILOWATT"] = "kW"
    SYMBOLS["MEGAWATT"] = "MW"
    SYMBOLS["MICROWATT"] = "µW"
    SYMBOLS["MILLIWATT"] = "mW"
    SYMBOLS["NANOWATT"] = "nW"
    SYMBOLS["PETAWATT"] = "PW"
    SYMBOLS["PICOWATT"] = "pW"
    SYMBOLS["TERAWATT"] = "TW"
    SYMBOLS["WATT"] = "W"
    SYMBOLS["YOCTOWATT"] = "yW"
    SYMBOLS["YOTTAWATT"] = "YW"
    SYMBOLS["ZEPTOWATT"] = "zW"
    SYMBOLS["ZETTAWATT"] = "ZW"

    def __init__(self, value=None, unit=None):
        _omero_model.Power.__init__(self)

        if unit is None:
            target = None
        elif isinstance(unit, UnitsPower):
            target = unit
        elif isinstance(unit, basestring):
            target = getattr(UnitsPower, unit)
        else:
            raise Exception("Unknown unit: %s (%s)" % (
                unit, type(unit)
            ))

        if isinstance(value, _omero_model.PowerI):
            # This is a copy-constructor call.

            source = value.getUnit()

            if target is None:
                raise Exception("Null target unit")
            if source is None:
                raise Exception("Null source unit")

            if target == source:
                self.setValue(value.getValue())
                self.setUnit(source)
            else:
                c = self.CONVERSIONS.get(source).get(target)
                if c is None:
                    t = (value.getValue(), source, target)
                    msg = "%s %s cannot be converted to %s" % t
                    raise Exception(msg)
                self.setValue(c(value.getValue()))
                self.setUnit(target)
        else:
            self.setValue(value)
            self.setUnit(target)

    def getUnit(self, current=None):
        return self._unit

    def getValue(self, current=None):
        return self._value

    def getSymbol(self, current=None):
        return self.SYMBOLS.get(str(self.getUnit()))

    @staticmethod
    def lookupSymbol(unit):
        return PowerI.SYMBOLS.get(str(unit))

    def setUnit(self, unit, current=None):
        self._unit = unit

    def setValue(self, value, current=None):
        self._value = value

    def __str__(self):
        return self._base_string(self.getValue(), self.getUnit())

_omero_model.PowerI = PowerI



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Node.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Session' not in _M_omero.model.__dict__:
    _M_omero.model._t_Session = IcePy.declareClass('::omero::model::Session')
    _M_omero.model._t_SessionPrx = IcePy.declareProxy('::omero::model::Session')

if 'NodeAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_NodeAnnotationLink = IcePy.declareClass('::omero::model::NodeAnnotationLink')
    _M_omero.model._t_NodeAnnotationLinkPrx = IcePy.declareProxy('::omero::model::NodeAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_NodeSessionsSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_NodeSessionsSeq = IcePy.defineSequence('::omero::model::NodeSessionsSeq', (), _M_omero.model._t_Session)

if '_t_NodeAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_NodeAnnotationLinksSeq = IcePy.defineSequence('::omero::model::NodeAnnotationLinksSeq', (), _M_omero.model._t_NodeAnnotationLink)

if '_t_NodeLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_NodeLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::NodeLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Node' not in _M_omero.model.__dict__:
    _M_omero.model.Node = Ice.createTempClass()
    class Node(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _sessionsSeq=None, _sessionsLoaded=False, _uuid=None, _conn=None, _up=None, _down=None, _scale=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.Node:
                raise RuntimeError('omero.model.Node is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._sessionsSeq = _sessionsSeq
            self._sessionsLoaded = _sessionsLoaded
            self._uuid = _uuid
            self._conn = _conn
            self._up = _up
            self._down = _down
            self._scale = _scale
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Node')

        def ice_id(self, current=None):
            return '::omero::model::Node'

        def ice_staticId():
            return '::omero::model::Node'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def unloadSessions(self, current=None):
            pass

        def sizeOfSessions(self, current=None):
            pass

        def copySessions(self, current=None):
            pass

        def addSession(self, target, current=None):
            pass

        def addAllSessionSet(self, targets, current=None):
            pass

        def removeSession(self, theTarget, current=None):
            pass

        def removeAllSessionSet(self, targets, current=None):
            pass

        def clearSessions(self, current=None):
            pass

        def reloadSessions(self, toCopy, current=None):
            pass

        def getUuid(self, current=None):
            pass

        def setUuid(self, theUuid, current=None):
            pass

        def getConn(self, current=None):
            pass

        def setConn(self, theConn, current=None):
            pass

        def getUp(self, current=None):
            pass

        def setUp(self, theUp, current=None):
            pass

        def getDown(self, current=None):
            pass

        def setDown(self, theDown, current=None):
            pass

        def getScale(self, current=None):
            pass

        def setScale(self, theScale, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addNodeAnnotationLink(self, target, current=None):
            pass

        def addAllNodeAnnotationLinkSet(self, targets, current=None):
            pass

        def removeNodeAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllNodeAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addNodeAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findNodeAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeNodeAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Node)

        __repr__ = __str__

    _M_omero.model.NodePrx = Ice.createTempClass()
    class NodePrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Node._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Node._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Node._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Node._op_setVersion.end(self, _r)

        def unloadSessions(self, _ctx=None):
            return _M_omero.model.Node._op_unloadSessions.invoke(self, ((), _ctx))

        def begin_unloadSessions(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_unloadSessions.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadSessions(self, _r):
            return _M_omero.model.Node._op_unloadSessions.end(self, _r)

        def sizeOfSessions(self, _ctx=None):
            return _M_omero.model.Node._op_sizeOfSessions.invoke(self, ((), _ctx))

        def begin_sizeOfSessions(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_sizeOfSessions.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfSessions(self, _r):
            return _M_omero.model.Node._op_sizeOfSessions.end(self, _r)

        def copySessions(self, _ctx=None):
            return _M_omero.model.Node._op_copySessions.invoke(self, ((), _ctx))

        def begin_copySessions(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_copySessions.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copySessions(self, _r):
            return _M_omero.model.Node._op_copySessions.end(self, _r)

        def addSession(self, target, _ctx=None):
            return _M_omero.model.Node._op_addSession.invoke(self, ((target, ), _ctx))

        def begin_addSession(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_addSession.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addSession(self, _r):
            return _M_omero.model.Node._op_addSession.end(self, _r)

        def addAllSessionSet(self, targets, _ctx=None):
            return _M_omero.model.Node._op_addAllSessionSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllSessionSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_addAllSessionSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllSessionSet(self, _r):
            return _M_omero.model.Node._op_addAllSessionSet.end(self, _r)

        def removeSession(self, theTarget, _ctx=None):
            return _M_omero.model.Node._op_removeSession.invoke(self, ((theTarget, ), _ctx))

        def begin_removeSession(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_removeSession.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeSession(self, _r):
            return _M_omero.model.Node._op_removeSession.end(self, _r)

        def removeAllSessionSet(self, targets, _ctx=None):
            return _M_omero.model.Node._op_removeAllSessionSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllSessionSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_removeAllSessionSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllSessionSet(self, _r):
            return _M_omero.model.Node._op_removeAllSessionSet.end(self, _r)

        def clearSessions(self, _ctx=None):
            return _M_omero.model.Node._op_clearSessions.invoke(self, ((), _ctx))

        def begin_clearSessions(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_clearSessions.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearSessions(self, _r):
            return _M_omero.model.Node._op_clearSessions.end(self, _r)

        def reloadSessions(self, toCopy, _ctx=None):
            return _M_omero.model.Node._op_reloadSessions.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadSessions(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_reloadSessions.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadSessions(self, _r):
            return _M_omero.model.Node._op_reloadSessions.end(self, _r)

        def getUuid(self, _ctx=None):
            return _M_omero.model.Node._op_getUuid.invoke(self, ((), _ctx))

        def begin_getUuid(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getUuid.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getUuid(self, _r):
            return _M_omero.model.Node._op_getUuid.end(self, _r)

        def setUuid(self, theUuid, _ctx=None):
            return _M_omero.model.Node._op_setUuid.invoke(self, ((theUuid, ), _ctx))

        def begin_setUuid(self, theUuid, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_setUuid.begin(self, ((theUuid, ), _response, _ex, _sent, _ctx))

        def end_setUuid(self, _r):
            return _M_omero.model.Node._op_setUuid.end(self, _r)

        def getConn(self, _ctx=None):
            return _M_omero.model.Node._op_getConn.invoke(self, ((), _ctx))

        def begin_getConn(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getConn.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getConn(self, _r):
            return _M_omero.model.Node._op_getConn.end(self, _r)

        def setConn(self, theConn, _ctx=None):
            return _M_omero.model.Node._op_setConn.invoke(self, ((theConn, ), _ctx))

        def begin_setConn(self, theConn, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_setConn.begin(self, ((theConn, ), _response, _ex, _sent, _ctx))

        def end_setConn(self, _r):
            return _M_omero.model.Node._op_setConn.end(self, _r)

        def getUp(self, _ctx=None):
            return _M_omero.model.Node._op_getUp.invoke(self, ((), _ctx))

        def begin_getUp(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getUp.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getUp(self, _r):
            return _M_omero.model.Node._op_getUp.end(self, _r)

        def setUp(self, theUp, _ctx=None):
            return _M_omero.model.Node._op_setUp.invoke(self, ((theUp, ), _ctx))

        def begin_setUp(self, theUp, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_setUp.begin(self, ((theUp, ), _response, _ex, _sent, _ctx))

        def end_setUp(self, _r):
            return _M_omero.model.Node._op_setUp.end(self, _r)

        def getDown(self, _ctx=None):
            return _M_omero.model.Node._op_getDown.invoke(self, ((), _ctx))

        def begin_getDown(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getDown.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDown(self, _r):
            return _M_omero.model.Node._op_getDown.end(self, _r)

        def setDown(self, theDown, _ctx=None):
            return _M_omero.model.Node._op_setDown.invoke(self, ((theDown, ), _ctx))

        def begin_setDown(self, theDown, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_setDown.begin(self, ((theDown, ), _response, _ex, _sent, _ctx))

        def end_setDown(self, _r):
            return _M_omero.model.Node._op_setDown.end(self, _r)

        def getScale(self, _ctx=None):
            return _M_omero.model.Node._op_getScale.invoke(self, ((), _ctx))

        def begin_getScale(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getScale.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getScale(self, _r):
            return _M_omero.model.Node._op_getScale.end(self, _r)

        def setScale(self, theScale, _ctx=None):
            return _M_omero.model.Node._op_setScale.invoke(self, ((theScale, ), _ctx))

        def begin_setScale(self, theScale, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_setScale.begin(self, ((theScale, ), _response, _ex, _sent, _ctx))

        def end_setScale(self, _r):
            return _M_omero.model.Node._op_setScale.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Node._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Node._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Node._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Node._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Node._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Node._op_copyAnnotationLinks.end(self, _r)

        def addNodeAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Node._op_addNodeAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addNodeAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_addNodeAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addNodeAnnotationLink(self, _r):
            return _M_omero.model.Node._op_addNodeAnnotationLink.end(self, _r)

        def addAllNodeAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Node._op_addAllNodeAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllNodeAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_addAllNodeAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllNodeAnnotationLinkSet(self, _r):
            return _M_omero.model.Node._op_addAllNodeAnnotationLinkSet.end(self, _r)

        def removeNodeAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Node._op_removeNodeAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeNodeAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_removeNodeAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeNodeAnnotationLink(self, _r):
            return _M_omero.model.Node._op_removeNodeAnnotationLink.end(self, _r)

        def removeAllNodeAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Node._op_removeAllNodeAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllNodeAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_removeAllNodeAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllNodeAnnotationLinkSet(self, _r):
            return _M_omero.model.Node._op_removeAllNodeAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Node._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Node._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Node._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Node._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Node._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Node._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Node._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Node._op_linkAnnotation.end(self, _r)

        def addNodeAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Node._op_addNodeAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addNodeAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_addNodeAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addNodeAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Node._op_addNodeAnnotationLinkToBoth.end(self, _r)

        def findNodeAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Node._op_findNodeAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findNodeAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_findNodeAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findNodeAnnotationLink(self, _r):
            return _M_omero.model.Node._op_findNodeAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Node._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Node._op_unlinkAnnotation.end(self, _r)

        def removeNodeAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Node._op_removeNodeAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeNodeAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_removeNodeAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeNodeAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Node._op_removeNodeAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Node._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Node._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Node._op_linkedAnnotationList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.NodePrx.ice_checkedCast(proxy, '::omero::model::Node', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.NodePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Node'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_NodePrx = IcePy.defineProxy('::omero::model::Node', NodePrx)

    _M_omero.model._t_Node = IcePy.declareClass('::omero::model::Node')

    _M_omero.model._t_Node = IcePy.defineClass('::omero::model::Node', Node, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_sessionsSeq', (), _M_omero.model._t_NodeSessionsSeq, False, 0),
        ('_sessionsLoaded', (), IcePy._t_bool, False, 0),
        ('_uuid', (), _M_omero._t_RString, False, 0),
        ('_conn', (), _M_omero._t_RString, False, 0),
        ('_up', (), _M_omero._t_RTime, False, 0),
        ('_down', (), _M_omero._t_RTime, False, 0),
        ('_scale', (), _M_omero._t_RInt, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_NodeAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    Node._ice_type = _M_omero.model._t_Node

    Node._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Node._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Node._op_unloadSessions = IcePy.Operation('unloadSessions', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Node._op_sizeOfSessions = IcePy.Operation('sizeOfSessions', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Node._op_copySessions = IcePy.Operation('copySessions', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_NodeSessionsSeq, False, 0), ())
    Node._op_addSession = IcePy.Operation('addSession', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Session, False, 0),), (), None, ())
    Node._op_addAllSessionSet = IcePy.Operation('addAllSessionSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeSessionsSeq, False, 0),), (), None, ())
    Node._op_removeSession = IcePy.Operation('removeSession', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Session, False, 0),), (), None, ())
    Node._op_removeAllSessionSet = IcePy.Operation('removeAllSessionSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeSessionsSeq, False, 0),), (), None, ())
    Node._op_clearSessions = IcePy.Operation('clearSessions', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Node._op_reloadSessions = IcePy.Operation('reloadSessions', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Node, False, 0),), (), None, ())
    Node._op_getUuid = IcePy.Operation('getUuid', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Node._op_setUuid = IcePy.Operation('setUuid', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Node._op_getConn = IcePy.Operation('getConn', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Node._op_setConn = IcePy.Operation('setConn', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Node._op_getUp = IcePy.Operation('getUp', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RTime, False, 0), ())
    Node._op_setUp = IcePy.Operation('setUp', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RTime, False, 0),), (), None, ())
    Node._op_getDown = IcePy.Operation('getDown', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RTime, False, 0), ())
    Node._op_setDown = IcePy.Operation('setDown', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RTime, False, 0),), (), None, ())
    Node._op_getScale = IcePy.Operation('getScale', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Node._op_setScale = IcePy.Operation('setScale', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Node._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Node._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Node._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_NodeAnnotationLinksSeq, False, 0), ())
    Node._op_addNodeAnnotationLink = IcePy.Operation('addNodeAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeAnnotationLink, False, 0),), (), None, ())
    Node._op_addAllNodeAnnotationLinkSet = IcePy.Operation('addAllNodeAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeAnnotationLinksSeq, False, 0),), (), None, ())
    Node._op_removeNodeAnnotationLink = IcePy.Operation('removeNodeAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeAnnotationLink, False, 0),), (), None, ())
    Node._op_removeAllNodeAnnotationLinkSet = IcePy.Operation('removeAllNodeAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeAnnotationLinksSeq, False, 0),), (), None, ())
    Node._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Node._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Node, False, 0),), (), None, ())
    Node._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Node._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_NodeAnnotationLink, False, 0), ())
    Node._op_addNodeAnnotationLinkToBoth = IcePy.Operation('addNodeAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Node._op_findNodeAnnotationLink = IcePy.Operation('findNodeAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_NodeAnnotationLinksSeq, False, 0), ())
    Node._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Node._op_removeNodeAnnotationLinkFromBoth = IcePy.Operation('removeNodeAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_NodeAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Node._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_NodeLinkedAnnotationSeq, False, 0), ())

    _M_omero.model.Node = Node
    del Node

    _M_omero.model.NodePrx = NodePrx
    del NodePrx

# End of module omero.model

__name__ = 'omero'

# End of module omero
